function [ B, succ ] = create_rand_matrices( A, k, varargin )
%CREATE_RAND_MATRICES Returns swap-randomized matrices
%   B = create_rand_matrices(A, k) returns k matrices of the same
%   size as A in cell array B.
%
%   CREATE_RAND_MATRICES(A, k, steps) sets the number of steps the chain
%   takes between every new sample (and in burn-in perioid). Default =
%   10000.
%
%   [B, succ] = CREATE_RAND_MATRICES() returns also the number of
%   successful swaps in a k-vector.
%
%   See also: SWAP_RAND

p = inputParser;
defSteps = 10000;
addRequired(p, 'A', @(x) validateattributes(x, {'numeric'}, {'square', 'binary'}, mfilename, 'A', 1));
addRequired(p, 'k', @(x) validateattributes(x, {'numeric'}, {'scalar', 'integer', 'positive'}, mfilename, 'k', 2));
addOptional(p, 'steps', defSteps, ...
    @(x) validateattributes(x, {'numeric'}, {'scalar', 'integer', 'positive'}, mfilename, 'steps', 3));
parse(p, A, k, varargin{:});

steps = p.Results.steps;

B = cell(k, 1);
succ = nan(k, 1);

%% Backwards
fprintf('Going backwards... ');
A = swap_rand(A, steps);
fprintf('done!\n');

parfor i=1:k
    fprintf('Sample %03i... ', i);
   [b, s] = swap_rand(A, steps);
   fprintf('done!\n');
   B{i} = b;
   succ(i) = s;
end

end

