function [ comm, LL, p1, p2 ] = growCommunity( commSeed, graph, paramType, maxSize )
%GROWCOMMUNITY expands a community from a given seed inside the graph
if ~exist('maxSize', 'var')
    maxSize = length(graph);
end

comm = commSeed;
[p1,p2,~] = recoverCommunity(full(graph(commSeed, commSeed)), paramType);
LL = computeLL(getCommunityStatisticsForPartialCommunity(...
               graph, [], p1,p2, getCondition(paramType))); %initially LL of all noise
LLDiff = 1;
epsilon = 0.0001;

LLdevelopment = [];
while LLDiff > log1p(epsilon) && length(comm) < maxSize
   [nodeToAdd, LLDiff, p1, p2, newLL] = getFirstGoodNodeToAdd(...
       comm, graph, paramType, LL);
    if nodeToAdd == 0
        disp('No additional node improves the LL.');
        newLL = LL;
        break;
    end
    LL = newLL;
    comm(end+1) = nodeToAdd;
    LLdevelopment(end+1,:) = [LL, LLDiff];
    fprintf('.');
end
fprintf('\n');
LL = newLL;

if false
    figure;[x,y]=find(degree_ord(graph(comm,comm)));
    area(0:length(comm)-1, sum(makeCommunity(length(comm), p1, p2, getCondition(paramType))));
    hold on; scatter(x-1,y-1);
    title(LL); axis equal;xlim([0 maxSize]);ylim([0 maxSize]); 
    figure;plot(LLdevelopment(:,1));hold on;plot(LLdevelopment(:,2));legend({'LL', 'LLDiff'});
end
end

function [nodeToAdd, maxLLdiff, p1new, p2new, bestLL] = getBestNodeToAdd(...
    comm, graph, paramType, currentLL)
    %take the neighboring node that improves the log-likelihood most
    connections = full(sum(graph(comm,:)));
    connections(comm) = 0;
    theNeighbors = find(connections > 0);
    N = length(theNeighbors);
    condition = getCondition(paramType);    
    neighborLLdiff = nan(1, N);
    neighborLL     = nan(1, N);
    neighborParam  = nan(2, N);
    
    for indexOfNeighbor = 1:N
       n = theNeighbors(indexOfNeighbor);
       newComm = [comm n];
       currentComm = full(graph(newComm, newComm)); 
       [p1new,p2new,~] = recoverCommunity(currentComm, paramType);
       LLnew = computeLL(getCommunityStatisticsForPartialCommunity(...
           graph, currentComm, p1new, p2new, condition));
       LLdiff = LLnew - currentLL;
       neighborLLdiff(indexOfNeighbor) = LLdiff;
       neighborLL(indexOfNeighbor) = LLnew;
       neighborParam(:,indexOfNeighbor) = [p1new,p2new];
    end
    % Solve ties by taking the node with most connections.
    maxLLdiff = max(neighborLLdiff);
    indicesOfBestNodes = neighborLLdiff == maxLLdiff;
    nodeIdx = find(connections(theNeighbors) == max(connections(theNeighbors(indicesOfBestNodes))),1);
    nodeToAdd = theNeighbors(nodeIdx);
    p1new = neighborParam(1,nodeIdx);
    p2new = neighborParam(2,nodeIdx);
    bestLL = neighborLL(nodeIdx);
end

function [nodeToAdd, LLdiff, p1new, p2new, LLnew] = getFirstGoodNodeToAdd(...
    comm, graph, paramType, currentLL)
    %take the most connected neighboring node that improves the log-likelihood
    nodeToAdd = 0;
    connections = full(sum(graph(comm,:)));
    connections(comm) = 0;
    condition = getCondition(paramType);    
    trials = 0;
    maxTrials = 100;
    
    while max(connections) > 0 && trials < maxTrials
       [~, candidate] = max(connections);
       connections(candidate) = 0;
       newComm = [comm candidate];
       currentComm = degree_ord(full(graph(newComm, newComm))); 
       [p1new,p2new,~] = recoverCommunity(currentComm, paramType);
       LLnew = computeLL(getCommunityStatisticsForPartialCommunity(...
           graph, currentComm, p1new, p2new, condition));
       LLdiff = LLnew - currentLL;
       if LLdiff > 0
           nodeToAdd = candidate;
           break;
       end
       trials = trials + 1;
    end
end






