minsize = 100; 
maxsize = 1000; 
dataName={'amazon', 'dblp', 'friendster','lj', 'orkut', 'youtube'};
hyperbolicLL = nan(length(dataName),1);
hyComLL = nan(length(dataName),1);
blockLL = nan(length(dataName),1);

parfor dataNameIndex = 1:length(dataName)
    tic
    d = char(dataName(dataNameIndex));
    disp(['--------------- ' d ' ---------------']);
    try
        dataset = load(['../data/' d '_' int2str(minsize) '_' int2str(Inf) '.mat']);
        try
            result = load(['../results/result_multiComm_'  d '_' int2str(minsize) '_' int2str(maxsize) '_1sample500_v2.mat']);
        catch
            disp('Alternative sample');
            result = load(['../results/result_multiComm_'  d '_' int2str(minsize) '_' int2str(maxsize) '_1sample500.mat']);
        end
        
        triuGraph = triu(dataset.graph);
        explainedEdges = sparse(length(dataset.graph),length(dataset.graph)); % if we use explainedEdges, we might end up with trying 
                                                                               % to model communities that have no edges at all 
        x = 0.5; % HyCom is like x,Sigma with x=0.5
        comms = repmat(struct('p1',x), length(dataset.commVec), 1 );
        blockComms = repmat(struct('p1',0), length(dataset.commVec), 1 );
        for l=1:length(result.order)
            comms(l).members = dataset.commVec{result.order(l)};
            [~, order] = degree_ord(triuGraph(comms(l).members, comms(l).members) & ~explainedEdges(comms(l).members,comms(l).members));    
            comms(l).members = comms(l).members(order);
            numEdges =length(find(triuGraph(comms(l).members, comms(l).members) & ~explainedEdges(comms(l).members, comms(l).members)));
            
            A = triuGraph(comms(l).members, comms(l).members) & ~explainedEdges(comms(l).members, comms(l).members);
            A = A-diag(diag(A)); %make sure the diagonal is empty
            N = length(A);
            numberOfEdges=length(find(triu(A)));
            edgeValue=zeros(numberOfEdges,1);
            if numberOfEdges > 0
                [indI,indJ]=find(triu(A));
            end
            for edge=1:numberOfEdges
                i=indI(edge)-1;
                j=indJ(edge)-1;
                edgeValue(edge)=(1-x)*(i*j)+(x)*(i+j);   % that are the Sigmas for every edge being the last one to include
            end
            lastCol = N-1;
            minSigma =(1-x)*(lastCol*0)+(x)*(lastCol+0);
            sigmas=sortrows(edgeValue);       %sort and exclude too small sigmas as
            sigmas = sigmas(sigmas>=minSigma);%we want to have the first col completely
            if isempty(sigmas)
                sigmas(1) = minSigma;
            end
            
            numElementsBelow = arrayfun(@(s) getElementsBelowXSigma(N,x,s), sigmas);
            numEdgesBelow=((numberOfEdges-length(sigmas)+1):numberOfEdges)';
            numZerosBelow=numElementsBelow-numEdgesBelow;
            % getElementsBelow might be off by 1 due to the floor function and
            % floating point arithmetics. We work around this by correcting the
            % area if we know that more edges should be inside than the calculated
            % area allows. In case this pragmatic approach does not work in some
            % particular case, the area can be computed alternatively with
            % getAreaOfExpectedCommunity which is much slower but precise.
            numZerosBelowNew=max(0,numZerosBelow);
            numElementsBelow=numElementsBelow+(numZerosBelowNew-numZerosBelow);
            numZerosBelow = numZerosBelowNew;
            numEdgesAbove=numberOfEdges-numEdgesBelow;   
            numZerosAbove=getAreaOfTriu(N)-numberOfEdges-numZerosBelow;
            ein=numEdgesBelow./numElementsBelow;
            eout=numEdgesAbove./(getAreaOfTriu(N)-numElementsBelow);
            %compute the LL
            loglike=logMult(numEdgesBelow,ein)+logMult(numZerosBelow,1-ein)...
                    +logMult(numEdgesAbove,eout)+logMult(numZerosAbove,1-eout);

            [maxLL,~]=max(loglike);
            pos = find(loglike==maxLL,1,'last'); % solve ties by taking the highest sigma
            bestSigma=sigmas(pos);

            comms(l).p2=bestSigma;
            comms(l).paramType='x,sigma';
            comms(l).model = makeCommunity(length(comms(l).members),comms(l).p1, comms(l).p2 , getCondition(comms(l).paramType));

            explainedEdges(comms(l).members,comms(l).members) = explainedEdges(comms(l).members,comms(l).members) ...
                                                        | (triuGraph(comms(l).members,comms(l).members) & comms(l).model);

            blockComms(l).p1=length(dataset.commVec{result.order(l)})-1;
            blockComms(l).p2=length(dataset.commVec{result.order(l)})-1;
            blockComms(l).paramType='H,gamma';
            blockComms(l).members = dataset.commVec{result.order(l)};
            blockComms(l).model = makeCommunity(length(blockComms(l).members),blockComms(l).p1, blockComms(l).p2 , getCondition(blockComms(l).paramType));
        end
        blockLL(dataNameIndex) = computeLLOfGraph(dataset.graph,blockComms);
        hyComLL(dataNameIndex) = computeLLOfGraph(dataset.graph,comms);
        hyperbolicLL(dataNameIndex) = computeLLOfGraph(dataset.graph,result.res);
        disp([d ':']);
        fprintf('=> LL of hyCom model %i\n',hyComLL(dataNameIndex));
        fprintf('=> LL of block model %i\n',blockLL(dataNameIndex));
        fprintf('=> LL of hyperbolic model %i\n',hyperbolicLL(dataNameIndex));

        %likelihood ratio test: D = 2 * (alternativeModel - nullModel)
        D = 2*(hyperbolicLL(dataNameIndex)-blockLL(dataNameIndex));
        fprintf('LL ratio block: %.1f\n',D);
        D = 2*(hyperbolicLL(dataNameIndex)-hyComLL(dataNameIndex));
        fprintf('LL ratio hyCom: %.1f\n',D);
    catch e
        disp(e.message);
    end
    toc     
end
LLratioBlock = 2*(hyperbolicLL-blockLL);
LLratioHyCom = 2*(hyperbolicLL-hyComLL);
save(['../results/result_LLs_hyCom_withSigmaFit_' int2str(minsize) '_' int2str(maxsize) '_1sample500_v2.mat'], 'hyComLL', 'hyperbolicLL', 'blockLL', 'LLratioBlock','LLratioHyCom','dataName');
disp('Done.');
diary off;
