dataset = {'polbooks', 'jazz', 'Erdos971', 'email'};
ks = [ 6 5 8 10];
minSize = 5;
for dd=1:4;
    d = char(dataset(dd));
    fprintf('\n\n******* Dataset: %s *********\n',d);
    load(['../data/' d '.mat'], 'Problem');X = Problem.A; % load graph (1-based index)
    M = importdata(['../data/' d '.csv']); % load communities (0-based index)
    M = M(2:end);   
    % The communities from HyCom come with a parameter alpha<=0 and a
    % mistake count as well as an edge count. In order to relate HyCom 
    % communities to ours, we need to derive the threshold sigma, 
    % s.t. ij(1-x)+(i+j)x<=sigma. We have x=0.5 in all HyCom communities 
    % and the relation sigma = 0.5*(tau^(1/alpha)-1).
    comms = repmat(struct('p1',0.5), 1, 1 ); % i.e. x = 0.5
    c = 0;
    for l = 1:ks(dd)%numel(M)
        parts = strsplit(char(M{l}),', ');
        alpha = str2num(parts{4});
        if alpha == 0
            alpha = -eps;
        end
        numMistakes = str2num(parts{3});
        numEdges = str2num(parts{2});
        comm = str2num(parts{5}(1:end-3));

        if length(comm) < minSize
            break;
        end
        c = c+1;
        comms(c).members = comm + 1; % switch to 1-based indexing
        [comms(c).comm, ord] = degree_ord(X(comms(c).members, comms(c).members));
        comms(c).members = comms(c).members(ord);
              
        A = ones(length(comm));
        A = A-diag(diag(A)); %make sure the diagonal is empty
        numberOfEdges=length(find(triu(A)));
        edgeValue=zeros(numberOfEdges,1);
        [indI,indJ]=find(triu(A));
        for edge=1:numberOfEdges
                i=indI(edge);
                j=indJ(edge);
                edgeValue(edge)=(i*j)^alpha; % HyCom model assumes 1-based indexing   
        end
        taus=sortrows(edgeValue);
        tau = taus(min(numEdges,numberOfEdges-1)); % That's the tau with which we include numEdges edges.
        
        comms(c).p1 = 0.5;
        comms(c).p2=0.5*(tau^(1/alpha)-1);
        comms(c).paramType='x,sigma';
        comms(c).model = makeCommunity(length(comms(c).members), comms(c).p1, comms(c).p2, getCondition(comms(c).paramType));
    end
    HyComLL = computeLLOfGraph(X,comms);
    fprintf('HyCom gave %d communities of size at least %d and yields an LL of %f\n', length(comms), minSize,HyComLL);
    
    %switch to H,gamma representation as that computes faster
    for c = 1:length(comms)
        p = getPFromX(comms(c).p1);
        theta = getThetaFromP(p, comms(c).p2);
        N = length(comms(c).members);
        H = getH(theta,p,N);
        gamma = getGamma(p,H,N);
        comms(c).paramType = 'H,gamma';
        comms(c).p1 = H;
        comms(c).p2 = gamma;
    end
    res=improveModel(X, comms, true);
    LL = computeLLOfGraph(X,res);
    fprintf('LL after model improvement: %f\n', LL);
    
    %switch to X,sigma representation to examine what changed
    for c = 1:length(res)
        N = length(res(c).members);
        p = getP(res(c).p2, N, res(c).p1);
        theta = getTheta(res(c).p2, N, res(c).p1);
        x = getX(p);
        sigma = getSigma(theta, p);
        res(c).paramType = 'x,sigma';
        res(c).p1 = x;
        res(c).p2 = sigma;
    end
    
    fprintf('LL ratio: %f\n', 2*(LL-HyComLL));
end